module TortoiseLabs
  
  # IPs
  
  class IP
    attr_reader :broadcast, :netmask, :version, :gateway, :network, :ip
    
    def initialize(iphash)
      @ip = iphash["ip"]
      ipnet = iphash["ipnet"]
      @netmask, @broadcast, @version, @gateway, @network = ipnet["netmask"],
        ipnet["broadcast"], ipnet["version"].to_i, ipnet["gateway"],
        ipnet["network"]
    end
    
    def to_s
      @ip
    end
  end
  
  # Jobs
  
  class Job
    attr_reader :id, :method, :entry_ts, :start_ts, :success
    
    def initialize(job)
      @id, @method, @entry_ts, @start_ts = job["id"], job["req_env"]["method"],
        job["entry_ts"], job["start_ts"]
        
      @success = job["rsp_env"]["params"]["success"]
    end
    
    def to_s
      "Job #{@id}: #{@method}"
    end
    
    def success?
      @success
    end
  end
  
  # VPS
        
  class VPS
    attr_reader :id, :disk, :mac, :memory, :name, :node
    attr_reader :swap, :user, :nickname, :ips
    
    # Instance methods
    
    def initialize(vps)
      @node, @name, @mac, @memory, @disk = vps["node"], vps["name"],  
        vps["mac"], vps["memory"], vps["disk"]
        
      @nickname, @id, @swap, @user = vps["nickname"], vps["id"], 
        vps["swap"], vps["user"]
        
      @ips = Hash.new
      vps["ips"].each do |iphash|
        obj = IP.new(iphash)
        @ips[obj.ip] = obj
      end
    end
    
    def to_s
      @nickname
    end
    
    def nickname=(newnickname)
      options = {:nickname => newnickname}
      result = TortoiseLabs::Client.post("/vps/#{@id}/setnickname",  options)
      @nickname = result["nickname"]
    end
    
    def monitoring=(boolean)
      if boolean
        result = TortoiseLabs::Client.get("/vps/#{@id}/monitoring/enable")
      else
        result = TortoiseLabs::Client.get("/vps/#{@id}/monitoring/disable")
      end
      boolean
    end
    
    def monitoring?
      vps = JSON.parse(TortoiseLabs::Client.get("/vps/#{id}"))
      vps["service"]["monitoring"]
    end
    
    def templates
      result = TortoiseLabs::Client.get("/vps/#{@id}/deploy")
      JSON.parse(result)["templates"]
    end
    
    def deploy(image, rootpass, arch = "x86_64")
      # arch can be either x86_64 or i386
      # Image is one of the values returned by the templates method above
      
      unless image =~ /_login\.xml$/i
        image = image + "_login.xml"
      end
      
      result = TortoiseLabs::Client.post("/vps/#{@id}/deploy", 
        {:imagename => image, :rootpass => rootpass, :arch => arch})
      jobs
    end
    
    def running?
      result = TortoiseLabs::Client.get("/vps/#{@id}/status.json")
      result.parsed_response["running"]
    end
    
    def destroy
      result = TortoiseLabs::Client.get("/vps/#{@id}/destroy")
      result.parsed_response["job"]
    end
    
    def shutdown
      result = TortoiseLabs::Client.get("/vps/#{@id}/shutdown")
      result.parsed_response["job"]
    end
    
    def create
      result = TortoiseLabs::Client.get("/vps/#{@id}/create")
      result.parsed_response["job"]
    end
    
    def powercycle
      result = TortoiseLabs::Client.get("/vps/#{@id}/powercycle")
      result.parsed_response["job"]
    end
    
    def jobs
      res = TortoiseLabs::Client.get("/vps/#{@id}/jobs.json").parsed_response
      
      jobs = Hash.new
      res.each do |jobhash|
        begin
          obj = Job.new(jobhash)
          jobs[obj.id] = obj
        rescue NoMethodError => err
          # occurs when there are no jobs
        end
      end
      
      jobs
    end
    
    def hvm
      result = TortoiseLabs::Client.get("/vps/#{@id}/hvm").parsed_response
      JSON.parse(result)["isolist"]
    end
    
    def set_iso(isoid)
      result = TortoiseLabs::Client.post("/vps/#{@id}/hvm/setiso", 
        {:isoid => isoid})
      hvm
    end
    
    def set_boot_order(bootorder)
      result = TortoiseLabs::Client.post("/vps/#{@id}/hvm/setbootorder",
        {:bootorder => bootorder})
      result.parsed_response["isolist"]
    end
    
    def set_nic_type(nictype)
      # nictype can be e1000, virtio-net or rtl8139
      result = TortoiseLabs::Client.post("/vps/#{@id}/hvm/setnictype",
        {:nictype => nictype})
      hvm
    end
    
    def new_iso(isoname, isouri)
      result = TortoiseLabs::Client.post("/vps/#{@id}/hvmiso/new", 
        {:isoname => isoname, :isouri => isouri})
      hvm
    end
    
    def delete_iso(isoid)
      result = TortoiseLabs::Client.post("/vps/#{@id}/hvmiso/#{isoid}/delete")
      hvm
    end
    
    # Class methods
    
    def self.list
      json = JSON.parse(TortoiseLabs::Client.get("/vps/list"))
      return nil if json.empty?
      
      vpses = Hash.new
      json["vpslist"].each do |vps|
        obj = self.new(vps)
        vpses[obj.nickname] = obj
      end
      
      vpses
    end

    def self.regions
      result = JSON.parse(TortoiseLabs::Client.get("/vps/signup"))["regions"]
      regions = Array.new
      result.each { |r| regions << Region.new(r["id"], r["name"]) }
      regions
    end
    
    def self.plans
      result = JSON.parse(TortoiseLabs::Client.get("/vps/signup"))
      plans = Array.new
      result["resource_plans"].each do |p|
        obj = Plan.new(p["id"], p["disk"], p["memory"], p["name"], 
          p["price_btc"], p["price_usd"], p["swap"])
        plans << obj
      end
      plans
    end
    
    def self.signup(regionid, planid)
      result = TortoiseLabs::Client.post("/vps/signup",
        {:region => regionid, :plan => planid})
    end
    
    def self.get(id)
      vpshash = JSON.parse(TortoiseLabs::Client.get("/vps/#{id}"))["service"]
      self.new(vpshash)
    end
  end
  
  # Helper structs
    
  Region = Struct.new(:id, :name)
    
  Plan = Struct.new(:id, :disk, :memory, :name, :price_btc,
                      :price_usd, :swap)

end
